/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.content.Context;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;
import android.util.NoSuchPropertyException;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.util.HashMap;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;

@RunWith(AndroidJUnit4.class)
public class MessageProcessorTest {
    private static final String TAG = MessageProcessorTest.class.getName();

    private Context context;
    private MessageProcessor messageProcessor;

    @Before
    public void initTargetContext() {
        context = InstrumentationRegistry.getTargetContext();
        messageProcessor = new MessageProcessor(context);
        assertThat(context, notNullValue());
        assertThat(messageProcessor, notNullValue());
        messageProcessor.notificationBuilder = mock(PushNotificationBuilder.class);
    }

    @Test
    public void processMessageMissingKey() {
        HashMap map = new HashMap<>();
        map.put(MessageProcessor.TITLE_KEY_DATA_PAYLOAD, "test title");
        map.put(MessageProcessor.MESSAGE_KEY_DATA_PAYLOAD, "test message");
        map.put(MessageProcessor.URL_KEY_DATA_PAYLOAD, "http://testurl.com");

        try {
            messageProcessor.processMessage(map);
            fail("Should not be reached. Missing expiry time key did not throw exception.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void processMessageInvalidBundle() {
        HashMap map = new HashMap<>();
        map.put("key1", "value1");
        map.put(MessageProcessor.MESSAGE_KEY_DATA_PAYLOAD, "Test Message");

        try {
            messageProcessor.processMessage(map);
            fail("Should not be reached. Invalid key did not throw exception.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void processMessageSuccessful() throws Exception {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String testExpiry = "1234567";
        String testMessageId = "testMessageId";

        HashMap map = new HashMap<>();
        map.put(MessageProcessor.TITLE_KEY_DATA_PAYLOAD, testTitle);
        map.put(MessageProcessor.MESSAGE_KEY_DATA_PAYLOAD, testMessage);
        map.put(MessageProcessor.URL_KEY_DATA_PAYLOAD, testUrl);
        map.put(MessageProcessor.EXPIRY_KEY_DATA_PAYLOAD, testExpiry);
        map.put(MessageProcessor.MESSAGE_ID_KEY_DATA_PAYLOAD, testMessageId);

        messageProcessor.processMessage(map);

        verify(messageProcessor.notificationBuilder).postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);
    }

    @Test
    public void retrieveDataFromBundleValidKey() {
        HashMap map = new HashMap<>();
        map.put("key1", "value1");
        map.put("key2", "value2");

        String value = MessageProcessor.retrieveFromDataMap(map, "key1");
        assertThat(value, equalTo("value1"));
    }

    @Test
    public void retrieveDataFromBundleInvalidKey() {
        HashMap map = new HashMap<>();
        map.put("key1", "value1");
        map.put("key2", "value2");

        try {
            String value = MessageProcessor.retrieveFromDataMap(map, "key3");
            fail("Should not be reached. Invalid key did not throw exception.");
        } catch (NoSuchPropertyException e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void retrieveDataFromBundleValidKeyForInteger() {
        HashMap map = new HashMap<>();
        map.put("key1", "value1");
        map.put("key2", 111222);

        try {
            String value = MessageProcessor.retrieveFromDataMap(map, "key2");
            fail("Should not be reached. Invalid key did not throw exception.");
        } catch (NoSuchPropertyException e) {
            assertThat(e, notNullValue());
            return;
        }
    }
}
