/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.provider.Settings;
import android.support.test.InstrumentationRegistry;
import android.support.test.filters.SdkSuppress;
import android.support.test.runner.AndroidJUnit4;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.ArgumentCaptor;

import java.util.List;
import java.util.concurrent.TimeUnit;

import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyInt;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;


@RunWith(AndroidJUnit4.class)
public class PushNotificationBuilderTest {

    private static final int TEST_RESOURCE_ID = 1234;
    private static final Class<?> EMPTY_ACTIVITY_CLASS = TestActivity.class;
    private Context context;
    private PushNotificationBuilder pushNotificationBuilder;
    private SharedPreferences sharedPref;

    private String twoHoursFromNowInSeconds() {
        long twoHoursFromNowInSeconds = TimeUnit.MILLISECONDS.toSeconds(System.currentTimeMillis()) + TimeUnit.HOURS.toSeconds(2);
        return String.valueOf(twoHoursFromNowInSeconds);
    }

    private String twoHoursAgoInSeconds() {
        long twoHoursFromNowInSeconds = TimeUnit.MILLISECONDS.toSeconds(System.currentTimeMillis()) - TimeUnit.HOURS.toSeconds(2);
        return String.valueOf(twoHoursFromNowInSeconds);
    }


    @Before
    public void initTargetContext() {
        context = InstrumentationRegistry.getTargetContext();
        pushNotificationBuilder = new PushNotificationBuilder(context);
        assertThat(context, notNullValue());

        sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        sharedPref.edit()
                .putInt(context.getString(R.string.icon_resource_id_pref_key), TEST_RESOURCE_ID)
                .putString(context.getString(R.string.activity_name_notification_selection_pref_key), EMPTY_ACTIVITY_CLASS.getName())
                .commit();
    }

    @Test
    public void createNotification() throws ClassNotFoundException {
        Notification result = pushNotificationBuilder.createNotification(
                "Title", "Contents!", "https://www.test.com", twoHoursFromNowInSeconds(), "test_message_id", 123);

        assertThat(result, notNullValue());
    }

    @Test
    @SdkSuppress(minSdkVersion = Build.VERSION_CODES.O)
    public void testNotificationChannel() {
        assertThat(pushNotificationBuilder.notificationChannel, notNullValue());
        assertEquals(pushNotificationBuilder.notificationChannel.getSound(), Settings.System.DEFAULT_NOTIFICATION_URI);
    }

    @Test
    public void postNotificationValidBundle() throws Exception {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String testExpiry = twoHoursFromNowInSeconds();
        String testMessageId = "test_message_id";

        pushNotificationBuilder.notificationManager = mock(NotificationManager.class);

        pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);
        verify(pushNotificationBuilder.notificationManager).notify(anyInt(), any(Notification.class));
    }

    @Test
    public void postNotificationActivityNameNotInSharedPref() {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String testExpiry = twoHoursFromNowInSeconds();
        String testMessageId = "test_message_id";

        sharedPref.edit()
                .remove(context.getString(R.string.activity_name_notification_selection_pref_key))
                .commit();

        try {
            pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);
            fail("Should not be reached. Invalid key did not throw exception.");
        } catch (ClassNotFoundException e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void postNotificationActivityNameInvalidInSharedPref() {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String testExpiry = twoHoursFromNowInSeconds();
        String testMessageId = "test_message_id";

        sharedPref.edit()
                .putString(context.getString(R.string.activity_name_notification_selection_pref_key), "blah")
                .commit();

        try {
            pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);
            fail("Should not be reached. Invalid key did not throw exception.");
        } catch (ClassNotFoundException e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void postNotificationMessageAlreadyExpired() throws Exception {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String alreadyExpired = twoHoursAgoInSeconds();
        String testMessageId = "test_message_id";

        pushNotificationBuilder.notificationManager = mock(NotificationManager.class);

        pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, alreadyExpired, testMessageId);
        verify(pushNotificationBuilder.notificationManager, never()).notify(anyInt(), any(Notification.class));
    }

    @Test
    public void postNotificationNotificationsHaveDifferentPendingIntentRequestIds() throws Exception {
        String testTitle = "test title";
        String testMessage = "test message";
        String testUrl = "http://testurl.com";
        String testExpiry = twoHoursFromNowInSeconds();
        String testMessageId = "test_message_id";

        pushNotificationBuilder.notificationManager = mock(NotificationManager.class);

        pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);
        pushNotificationBuilder.postNotification(testTitle, testMessage, testUrl, testExpiry, testMessageId);

        ArgumentCaptor<Notification> argumentCaptor1 = ArgumentCaptor.forClass(Notification.class);
        verify(pushNotificationBuilder.notificationManager, times(2)).notify(anyInt(), argumentCaptor1.capture());

        List<Notification> notifications = argumentCaptor1.getAllValues();

        Notification notification1 = notifications.get(0);
        Notification notification2 = notifications.get(1);

        PendingIntent pendingIntent1 = notification1.contentIntent;
        PendingIntent pendingIntent2 = notification2.contentIntent;
        assertFalse(pendingIntent1.equals(pendingIntent2));
    }
}
