/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.support.test.InstrumentationRegistry;
import android.support.test.annotation.UiThreadTest;
import android.support.test.filters.MediumTest;
import android.support.test.rule.ActivityTestRule;
import android.support.test.runner.AndroidJUnit4;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;

import org.json.JSONObject;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.CoreMatchers.nullValue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyInt;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(AndroidJUnit4.class)
@MediumTest
public class PushRegistrationManagerTest {
    private static final String TEST_SITE_ID = "someSiteId";
    private static final int TEST_RESOURCE_ID = 1234;
    private static final String TEST_SANDY_CLIENT_ID = "1234567890abcdef";
    private static final String TEST_ACTIVITY_NAME = TestActivity.class.getName();
    private static final String TEST_LOCALE = "en_CA";
    private static final Boolean TEST_OPTED_OUT = true;

    private Context context;
    private PushRegistrationManager pushRegistrationManager;
    private PushRegistrationManager.PushEventListener pushEventListener;
    private SharedPreferences sharedPref;

    @Rule
    public ActivityTestRule<TestActivity> testActivityRule = new ActivityTestRule(TestActivity.class);


    @Before
    public void setUp() throws Exception {
        context = InstrumentationRegistry.getTargetContext();

        pushEventListener = mock(PushRegistrationManager.PushEventListener.class);

        sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        sharedPref.edit().clear().commit();

        pushRegistrationManager = new PushRegistrationManager(context, TEST_ACTIVITY_NAME, pushEventListener);
        assertThat(context, notNullValue());
    }

    @Test
    public void testActivityNameClientIdSavedToSharedPref() {
        sharedPref.edit().clear().commit();

        String activityNameForNotificationSelectionKey =
                context.getString(R.string.activity_name_notification_selection_pref_key);
        String savedActivityNameForNotificationSelection = sharedPref.getString(activityNameForNotificationSelectionKey, null);
        assertThat(savedActivityNameForNotificationSelection, nullValue());

        String clientIdKey =
                context.getString(R.string.pusheen_client_id_pref_key);
        String savedClientId = sharedPref.getString(clientIdKey, null);
        assertThat(savedClientId, nullValue());

        pushRegistrationManager = new PushRegistrationManager(context, TEST_ACTIVITY_NAME, pushEventListener);

        String postConstructionActivityNameForNotificationSelection = sharedPref.getString(activityNameForNotificationSelectionKey, null);
        String postContructionClientId = sharedPref.getString(clientIdKey, null);

        assertThat(postConstructionActivityNameForNotificationSelection, equalTo(TEST_ACTIVITY_NAME));
        assertThat(postContructionClientId, notNullValue());
        assertThat(postContructionClientId.length(), equalTo(PushRegistrationManager.CLIENT_ID_LENGTH_HEX_DIGITS));
    }

    @Test
    public void testSetSiteId() throws Exception {
        String siteIdKey = context.getString(R.string.pusheen_site_id_pref_key);
        String savedSiteId = sharedPref.getString(siteIdKey, null);
        assertThat(savedSiteId, nullValue());

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);

        String postConstructionSiteId = sharedPref.getString(siteIdKey, null);
        assertThat(postConstructionSiteId, equalTo(TEST_SITE_ID));
    }

    @Test
    public void testSetIconResourceId() throws Exception {
        String iconResourceIdKey =
                context.getString(R.string.icon_resource_id_pref_key);
        int savedIconResourceId = sharedPref.getInt(iconResourceIdKey, 0);
        assertThat(savedIconResourceId, equalTo(0));

        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);

        int postConstructionIconResourceId = sharedPref.getInt(iconResourceIdKey, 0);
        assertThat(postConstructionIconResourceId, equalTo(TEST_RESOURCE_ID));
    }

    @Test
    public void testSetLocale() throws Exception {
        String localeKey = context.getString(R.string.locale_pref_key);
        String savedLocale = sharedPref.getString(localeKey, null);
        assertThat(savedLocale, nullValue());

        pushRegistrationManager.setLocale(TEST_LOCALE);

        String postConstructionLocale = sharedPref.getString(localeKey, null);
        assertThat(postConstructionLocale, equalTo(TEST_LOCALE));
    }

    @Test
    public void testSetOptedOut() throws Exception {
        String optedOutKey = context.getString(R.string.opted_out_pref_key);
        Boolean savedOptedOut = sharedPref.getBoolean(optedOutKey, false);
        assertThat(savedOptedOut.toString(), equalTo(Boolean.FALSE.toString()));

        pushRegistrationManager.setOptedOut(TEST_OPTED_OUT);

        Boolean postConstructionOptedOut = sharedPref.getBoolean(optedOutKey, false);
        assertThat(postConstructionOptedOut.toString(), equalTo(TEST_OPTED_OUT.toString()));
    }

    @Test
    public void testEnablePushMultipleTimesOnlyCallsbackOnce() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        Activity activity = mock(Activity.class);

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);

        pushRegistrationManager.enablePushNotifications(activity);
        pushRegistrationManager.enablePushNotifications(activity);
        pushRegistrationManager.enablePushPreviewNotifications(activity);

        verify(pushEventListener, times(1)).systemAllowed(any(String.class));
    }

    @Test
    public void testEnablePushGooglePlayServicesEnabledAppEnabledPushSettingSaved() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        Activity activity = mock(Activity.class);

        String appEnabledPushKey = context.getString(R.string.app_enabled_push_pref_key);

        SharedPreferences sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        sharedPref.edit().remove(appEnabledPushKey).commit();

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        pushRegistrationManager.enablePushNotifications(activity);

        boolean appEnabledPushResult = sharedPref.getBoolean(appEnabledPushKey, false);
        assertTrue(appEnabledPushResult);

        verify(pushEventListener).systemAllowed(any(String.class));
    }

    @UiThreadTest
    public void testEnablePushGooglePlayServicesUserResolvableTest() throws Exception {
        pushRegistrationManager.apiAvailability = spy(pushRegistrationManager.apiAvailability);
        doReturn(ConnectionResult.SERVICE_DISABLED).when(pushRegistrationManager.apiAvailability).isGooglePlayServicesAvailable(any(Context.class));

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());
        verify(pushEventListener).systemBlocked(any(String.class));
        verify(pushRegistrationManager.apiAvailability).getErrorDialog(any(Activity.class), anyInt(), anyInt());
    }

    @Test
    public void testEnablePushNoSiteIdOrIconResourceIdSetThrowsException() {
        try {
            pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());
            fail("Should not be reached.");
        } catch (Exception e) {
            assertNotNull(e);
            assertTrue(e instanceof PushRegistrationManager.PushException);
        }
    }

    @Test
    public void testEnablePushNoSiteIdSetThrowsException() {
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        try {
            pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());
            fail("Should not be reached.");
        } catch (Exception e) {
            assertNotNull(e);
            assertTrue(e instanceof PushRegistrationManager.PushException);
        }
    }

    @Test
    public void testEnablePushNoResourceIdSetThrowsException() {
        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        try {
            pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());
            fail("Should not be reached.");
        } catch (Exception e) {
            assertNotNull(e);
            assertTrue(e instanceof PushRegistrationManager.PushException);
        }
    }

    @Test
    public void testDeviceGooglePlayServicesCompatibleIsCompatible() {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        boolean result = pushRegistrationManager.deviceGooglePlayServicesCompatible();

        assertTrue(result);
    }

    @Test
    public void testDeviceGooglePlayServicesCompatibleIsCompatibleWithInstall() {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SERVICE_DISABLED);

        boolean result = pushRegistrationManager.deviceGooglePlayServicesCompatible();

        assertTrue(result);
    }

    @Test
    public void testDeviceGooglePlayServicesCompatibleNotCompatible() {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.API_UNAVAILABLE);

        boolean result = pushRegistrationManager.deviceGooglePlayServicesCompatible();

        assertFalse(result);
    }

    @Test
    public void testGetSubscriptionStatusCanSubscribeTrue() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean canSubscribe = subscriptionStatus.getBoolean(PushRegistrationManager.CAN_SUBSCRIBE_KEY);
        assertTrue(canSubscribe);
    }

    @Test
    public void testGetSubscriptionStatusCanSubscribeFalse() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.API_UNAVAILABLE);

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean canSubscribe = subscriptionStatus.getBoolean(PushRegistrationManager.CAN_SUBSCRIBE_KEY);
        assertFalse(canSubscribe);
    }

    @Test
    public void testGetSubscriptionStatusCanSubscribeFalseWhenAlreadySubscribed() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean canSubscribe = subscriptionStatus.getBoolean(PushRegistrationManager.CAN_SUBSCRIBE_KEY);
        assertFalse(canSubscribe);
    }

    @Test
    public void testGetSubscriptionStatusIsSubscribedTrue() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        pushRegistrationManager.enablePushNotifications(testActivityRule.getActivity());

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean isSubscribed = subscriptionStatus.getBoolean(PushRegistrationManager.IS_SUBSCRIBED_KEY);
        assertTrue(isSubscribed);
    }

    @Test
    public void testGetSubscriptionStatusIsSubscribedInPreviewModeReturnsTrue() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        pushRegistrationManager.setPusheenSiteId(TEST_SITE_ID);
        pushRegistrationManager.setPusheenSandyClientId(TEST_SANDY_CLIENT_ID);
        pushRegistrationManager.setNotificationIconResourceId(TEST_RESOURCE_ID);
        pushRegistrationManager.enablePushPreviewNotifications(testActivityRule.getActivity());

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean isSubscribed = subscriptionStatus.getBoolean(PushRegistrationManager.IS_SUBSCRIBED_KEY);
        assertTrue(isSubscribed);
    }

    @Test
    public void testGetSubscriptionStatusIsSubscribedFalse() throws Exception {
        pushRegistrationManager.apiAvailability = mock(GoogleApiAvailability.class);
        when(pushRegistrationManager.apiAvailability.isGooglePlayServicesAvailable(any(Context.class))).thenReturn(ConnectionResult.SUCCESS);

        JSONObject subscriptionStatus = pushRegistrationManager.getSubscriptionStatus();
        boolean isSubscribed = subscriptionStatus.getBoolean(PushRegistrationManager.IS_SUBSCRIBED_KEY);
        assertFalse(isSubscribed);
    }
}
