/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.content.Context;
import android.content.SharedPreferences;
import android.support.test.InstrumentationRegistry;
import android.support.test.runner.AndroidJUnit4;

import com.mobify.push.pushclient.rest.Attributes;
import com.mobify.push.pushclient.rest.PushSubscription;
import com.mobify.push.pushclient.rest.PusheenClient;
import com.mobify.push.pushclient.utilities.LocaleUtilities;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.IOException;
import java.lang.reflect.Constructor;

import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.mockito.Matchers.any;
import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;

@RunWith(AndroidJUnit4.class)

public class PusheenRegistrarTest {
    private static final String TAG = PusheenRegistrarTest.class.getName();

    private Context context;
    private PusheenRegistrar pusheenRegistrar;
    private SharedPreferences sharedPref;

    @Before
    public void initTargetContext() {
        context = InstrumentationRegistry.getTargetContext();
        pusheenRegistrar = new PusheenRegistrar(context);
        assertThat(context, notNullValue());
        assertThat(pusheenRegistrar, notNullValue());

        sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        sharedPref.edit().clear().commit();
        sharedPref.edit()
                .putString(context.getString(R.string.pusheen_site_id_pref_key), "testSiteId")
                .putBoolean(context.getString(R.string.app_enabled_preview_pref_key), false)
                .putString(context.getString(R.string.pusheen_client_id_pref_key), "0123456789ABCDEF")
                .putString(context.getString(R.string.pusheen_sandy_client_id_pref_key), "testSandyClient").commit();
    }

    private Call<Void> createSuccessfulMockCall() {
        return new Call<Void>() {
            @Override
            public Response<Void> execute() throws IOException {
                Void v = null;
                try {
                    Constructor<Void> constructor = Void.class.getDeclaredConstructor();
                    constructor.setAccessible(true);
                    v = constructor.newInstance();
                } catch (Exception e) {
                    fail("Failed building Void object.");
                }

                return Response.success(v);
            }

            @Override
            public void enqueue(Callback<Void> callback) {

            }

            @Override
            public boolean isExecuted() {
                return false;
            }

            @Override
            public void cancel() {

            }

            @Override
            public boolean isCanceled() {
                return false;
            }

            @Override
            public Call<Void> clone() {
                return null;
            }

            @Override
            public Request request() {
                return null;
            }
        };
    }

    private Call<Void> createErrorMockCall() {
        return new Call<Void>() {
            @Override
            public Response<Void> execute() throws IOException {
                ResponseBody responseBody = ResponseBody.create(MediaType.parse("application/json"), "BAD REQUEST");
                return Response.error(400, responseBody);
            }

            @Override
            public void enqueue(Callback<Void> callback) {

            }

            @Override
            public boolean isExecuted() {
                return false;
            }

            @Override
            public void cancel() {

            }

            @Override
            public boolean isCanceled() {
                return false;
            }

            @Override
            public Call<Void> clone() {
                return null;
            }

            @Override
            public Request request() {
                return null;
            }
        };
    }

    @Test
    public void registerTokenSuccessful() throws Exception {
        Call<Void> mockCall = createSuccessfulMockCall();

        pusheenRegistrar.pusheenClient = mock(PusheenClient.class);
        doReturn(mockCall).when(pusheenRegistrar.pusheenClient).updatePushSubscription(anyString(), anyString(), any(PushSubscription.class));


        String setToken = "ABCDEFG";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();

        try {
            pusheenRegistrar.registerToken();
        } catch (Exception e) {
            fail("Exception should not be thrown for successful request: " + e.getMessage());
        }

        boolean pusheenPreviewStatus = sharedPref.getBoolean(context.getString(R.string.pusheen_preview_enabled_pref_key), true);
        assertFalse(pusheenPreviewStatus);
    }

    @Test
    public void registerTokenPreviewEnabledSuccessful() throws Exception {
        Call<Void> mockCall = createSuccessfulMockCall();

        pusheenRegistrar.pusheenClient = mock(PusheenClient.class);
        doReturn(mockCall).when(pusheenRegistrar.pusheenClient).updatePushSubscription(anyString(), anyString(), any(PushSubscription.class));

        String setToken = "ABCDEFG";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();
        sharedPref.edit().putBoolean(context.getString(R.string.app_enabled_preview_pref_key), true).commit();

        try {
            pusheenRegistrar.registerToken();
        } catch (Exception e) {
            fail("Exception should not be thrown for successful request: " + e.getMessage());
        }

        boolean pusheenPreviewStatus = sharedPref.getBoolean(context.getString(R.string.pusheen_preview_enabled_pref_key), false);
        assertTrue(pusheenPreviewStatus);
    }

    @Test
    public void registerTokenNoGcmTokenSet() {
        sharedPref.edit().remove(context.getString(R.string.fcm_token_pref_key)).commit();

        try {
            pusheenRegistrar.registerToken();
            fail("Should not be reached. Exception expected when no gcm token is saved in shared preferences.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void registerTokenBadRequestFromServerThrowsIoException() {
        Call<Void> mockCall = createErrorMockCall();

        pusheenRegistrar.pusheenClient = mock(PusheenClient.class);
        doReturn(mockCall).when(pusheenRegistrar.pusheenClient).updatePushSubscription(anyString(), anyString(), any(PushSubscription.class));

        String setToken = "ABCDEFG";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();

        try {
            pusheenRegistrar.registerToken();
            fail("Should not be reached. Exception expected when 400 returned from server.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            assertTrue(e instanceof IOException);
            return;
        }
    }

    @Test
    public void registerTokenExceptionThrownClientIdNotSet() throws Exception {
        Call<Void> mockCall = createSuccessfulMockCall();

        pusheenRegistrar.pusheenClient = mock(PusheenClient.class);
        doReturn(mockCall).when(pusheenRegistrar.pusheenClient).updatePushSubscription(anyString(), anyString(), any(PushSubscription.class));

        String setToken = "ABCDEFG";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();
        sharedPref.edit().remove(context.getString(R.string.pusheen_client_id_pref_key)).commit();

        try {
            pusheenRegistrar.registerToken();
            fail("Should not be reached.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            return;
        }
    }

    @Test
    public void createPushSubscription() throws Exception{
        String setToken = "ABCDEFG";
        String sandyToken = "TEST_SANDY";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();
        sharedPref.edit().putString(context.getString(R.string.pusheen_sandy_client_id_pref_key), sandyToken).commit();

        PushSubscription pushSubscription = pusheenRegistrar.createPushSubscription();

        assertEquals("AndroidPushNotificationsNative", pushSubscription.endpoint);
        assertEquals(setToken, pushSubscription.subscription_id);
        assertEquals(false, pushSubscription.opted_out);

        Attributes attributes = pushSubscription.attributes;

        assertEquals(false, attributes.preview);
        assertEquals("user-agent", attributes.navigator_user_agent);
        assertEquals("subscribed", attributes.subscription_status);
        assertEquals(sandyToken, attributes.sandy_client_id);

        // Results could vary depending on system settings for the computer running the tests
        // That is why LocaleUtilities methods are used b/c they use the system default
        // behaviors
        assertEquals(LocaleUtilities.getDeviceLanguageAndCountry(), attributes.navigator_language);
        assertEquals(LocaleUtilities.getDeviceLanguageAndCountry(), attributes.locale);
        assertEquals(LocaleUtilities.getUtcOffsetInMinutes(), attributes.utc_offset_minutes);
    }

    @Test
    public void createPushSubscriptionWithPreviewEnabledAndLocalePreset() throws Exception{
        String setToken = "ABCDEFG";
        String testLocale = "de_DE";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken)
                .putBoolean(context.getString(R.string.app_enabled_preview_pref_key), true)
                .putString(context.getString(R.string.locale_pref_key), testLocale).commit();

        PushSubscription pushSubscription = pusheenRegistrar.createPushSubscription();

        assertEquals("AndroidPushNotificationsNative", pushSubscription.endpoint);
        assertEquals(setToken, pushSubscription.subscription_id);
        assertEquals(false, pushSubscription.opted_out);

        Attributes attributes = pushSubscription.attributes;

        assertEquals(true, attributes.preview);
        assertEquals("user-agent", attributes.navigator_user_agent);
        assertEquals("subscribed", attributes.subscription_status);

        // Results could vary depending on system settings for the computer running the tests
        // That is why LocaleUtilities methods are used b/c they use the system default
        // behaviors
        assertEquals(LocaleUtilities.getDeviceLanguageAndCountry(), attributes.navigator_language);
        assertEquals(testLocale, attributes.locale);
        assertEquals(LocaleUtilities.getUtcOffsetInMinutes(), attributes.utc_offset_minutes);
    }

    @Test
    public void registerTokenNoSiteIdSetThrowsException() {
        Call<Void> mockCall = createSuccessfulMockCall();

        pusheenRegistrar.pusheenClient = mock(PusheenClient.class);
        doReturn(mockCall).when(pusheenRegistrar.pusheenClient).updatePushSubscription(anyString(), anyString(), any(PushSubscription.class));

        String setToken = "ABCDEFG";
        sharedPref.edit().putString(context.getString(R.string.fcm_token_pref_key), setToken).commit();

        // Remove the site id
        sharedPref.edit().remove(context.getString(R.string.pusheen_site_id_pref_key)).commit();

        try {
            pusheenRegistrar.registerToken();
            fail("Registering token without a site id set should throw exception.");
        } catch (Exception e) {
            assertThat(e, notNullValue());
            return;
        }
    }
}
