/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.os.Build;
import android.provider.Settings;
import android.support.v4.app.NotificationCompat;
import android.util.Log;

import java.util.Random;
import java.util.concurrent.TimeUnit;

public class PushNotificationBuilder {
    private static final String TAG = PushNotificationBuilder.class.getName();

    public static final String CLIENT_ID_NOTIFICATION_EXTRA = "clientId";
    public static final String TITLE_NOTIFICATION_EXTRA = "title";
    public static final String MESSAGE_NOTIFICATION_EXTRA = "message";
    public static final String URL_NOTIFICATION_EXTRA = "url";
    public static final String MESSAGE_ID_NOTIFICATION_EXTRA = "messageId";
    public static final String CHANNEL_ID = "1";
    public static final String CHANNEL_NAME = "Push Notification";


    protected Context context;
    protected NotificationManager notificationManager;
    protected NotificationChannel notificationChannel;

    public PushNotificationBuilder(Context context) {
        this.context = context;
        this.notificationManager =
                (NotificationManager) this.context.getSystemService(Context.NOTIFICATION_SERVICE);
        this.createChannel();
    }

    private void createChannel() {
        if(Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            this.notificationChannel = new NotificationChannel(CHANNEL_ID, CHANNEL_NAME, NotificationManager.IMPORTANCE_DEFAULT);
            // TODO: Customize channel
            notificationManager.createNotificationChannel(notificationChannel);
        }
    }

    public void postNotification(String messageTitle, String messageContents, String messageUrlLink, String messageExpiry, String messageId) throws ClassNotFoundException {
        long timeUntilExpirySeconds = calculateTimeUntilExpiryMilliSeconds(messageExpiry);

        if (timeUntilExpirySeconds < 0) {
            Log.d(TAG, "Message has already expired.");
            return;
        }

        int notificationId = generateNotificationId();
        Notification notification =
                createNotification(messageTitle, messageContents, messageUrlLink, messageExpiry, messageId, notificationId);

        notificationManager.notify(notificationId, notification);
    }

    protected Notification createNotification(String messageTitle, String messageContents, String messageUrlLink, String messageExpiry, String messageId, int notificationId) throws ClassNotFoundException {
        SharedPreferences sharedPreferences = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);

        Intent intent = new Intent(context, getActivityForNotificationSelection(sharedPreferences));

        intent.putExtra(
                CLIENT_ID_NOTIFICATION_EXTRA,
                sharedPreferences.getString(context.getString(R.string.pusheen_client_id_pref_key), ""));

        intent.putExtra(TITLE_NOTIFICATION_EXTRA, messageTitle);
        intent.putExtra(MESSAGE_NOTIFICATION_EXTRA, messageContents);
        intent.putExtra(URL_NOTIFICATION_EXTRA, messageUrlLink);
        intent.putExtra(MESSAGE_ID_NOTIFICATION_EXTRA, messageId);

        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        PendingIntent pendingIntent = PendingIntent.getActivity(context, notificationId,
                intent, PendingIntent.FLAG_UPDATE_CURRENT);

        long timeUntilExpirySeconds = calculateTimeUntilExpiryMilliSeconds(messageExpiry);


        NotificationCompat.Builder builder = new NotificationCompat.Builder(context, CHANNEL_ID)
                .setSmallIcon(sharedPreferences.getInt(context.getString(R.string.icon_resource_id_pref_key), 0))
                .setContentTitle(messageTitle)
                .setContentText(messageContents)
                .setAutoCancel(true)
                .setContentIntent(pendingIntent)
                .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                .setStyle(new NotificationCompat.BigTextStyle()
                        .bigText(messageContents))
                .setTimeoutAfter(timeUntilExpirySeconds)
                .setSound(Settings.System.DEFAULT_NOTIFICATION_URI);

        return builder.build();
    }

    protected int generateNotificationId() {
        Random random = new Random();
        return Math.abs(random.nextInt());
    }

    private Class<?> getActivityForNotificationSelection(SharedPreferences sharedPreferences) throws ClassNotFoundException {
        String activityName =
                sharedPreferences.getString(context.getString(R.string.activity_name_notification_selection_pref_key), "");

        if (activityName.isEmpty()) {
            throw new ClassNotFoundException("Activity Name for Notification Selection was not set in SharedPreferences.");
        }

        return Class.forName(activityName);
    }

    private long calculateTimeUntilExpiryMilliSeconds(String messageExpiry) {
        long expiryTimeInSeconds = Long.parseLong(messageExpiry);
        long timeDifferenceInMilliSeconds = TimeUnit.SECONDS.toMillis(expiryTimeInSeconds) - System.currentTimeMillis();

        Log.d(TAG, "Expiry time seconds: " + expiryTimeInSeconds + " system utc time millis: " + System.currentTimeMillis() + " diff in milliseconds: " + timeDifferenceInMilliSeconds);
        return timeDifferenceInMilliSeconds;
    }
}
