/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GoogleApiAvailability;
import com.google.firebase.messaging.FirebaseMessaging;
import com.mobify.push.pushclient.service.PusheenRegistrationWorker;
import com.mobify.push.pushclient.utilities.HexUtilities;

import org.json.JSONException;
import org.json.JSONObject;

import androidx.work.WorkManager;

public class PushRegistrationManager {
    private static final String TAG = PushRegistrationManager.class.getName();

    public class PushException extends Exception {
        public PushException(String message) {
            super(message);
        }
    }

    public interface PushEventListener {
        void systemAllowed(String clientId);
        void systemBlocked(String clientId);
    }

    private static final int PLAY_SERVICES_RESOLUTION_REQUEST = 9000;
    protected static final int CLIENT_ID_LENGTH_HEX_DIGITS = 16;
    public static final String CAN_SUBSCRIBE_KEY = "canSubscribe";
    public static final String IS_SUBSCRIBED_KEY = "isSubscribed";
    public static final String OPTED_OUT_KEY = "optedOut";

    protected Context context;
    private PushEventListener pushEventListener;
    protected GoogleApiAvailability apiAvailability;
    protected SharedPreferences sharedPref;
    protected String clientId;

    public PushRegistrationManager(Context context,
                                   String activityNameForNotificationSelection,
                                   PushEventListener pushEventListener) {
        this.context = context;
        this.pushEventListener = pushEventListener;
        this.apiAvailability = GoogleApiAvailability.getInstance();

        // Persist notification service package name
        this.sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        sharedPref.edit()
                .putString(context.getString(R.string.activity_name_notification_selection_pref_key), activityNameForNotificationSelection)
                .apply();

        this.clientId = getClientId();
    }

    public void setPusheenSiteId(String pusheenSiteId) {
        sharedPref.edit().putString(context.getString(R.string.pusheen_site_id_pref_key), pusheenSiteId).commit();
    }

    public void setPusheenSandyClientId(String sandyClientId) {
        sharedPref.edit().putString(context.getString(R.string.pusheen_sandy_client_id_pref_key), sandyClientId).commit();
    }

    public void setNotificationIconResourceId(int notificationIconResourceId) {
        sharedPref.edit().putInt(context.getString(R.string.icon_resource_id_pref_key), notificationIconResourceId).commit();
    }

    public void setLocale(String locale) {
        sharedPref.edit().putString(context.getString(R.string.locale_pref_key), locale).commit();

        // Locale can be updated after the registration has occurred.
        // In this case pusheen needs to be notified with the update

        boolean appEnabledPush = sharedPref.getBoolean(context.getString(R.string.app_enabled_push_pref_key), false);
        String fcmToken = sharedPref.getString(context.getString(R.string.fcm_token_pref_key), "");

        // Can only update pusheen if app push is enabled and the fcm token has been retrieved
        // Otherwise if the token hasn't yet been retrieve,
        // then the first registration with pusheen hasn't occurred so the locale update should occur at that point
        if (appEnabledPush && !fcmToken.isEmpty()) {
            WorkManager.getInstance().enqueue(PusheenRegistrationWorker.createTask());
        }
    }

    public void setOptedOut(Boolean optedOut) {
        sharedPref.edit().putBoolean(context.getString(R.string.opted_out_pref_key), optedOut).commit();

        // Opted out can be updated after the registration has occurred.
        // In this case pusheen needs to be notified with the update

        boolean appEnabledPush = sharedPref.getBoolean(context.getString(R.string.app_enabled_push_pref_key), false);
        String fcmToken = sharedPref.getString(context.getString(R.string.fcm_token_pref_key), "");

        // Can only update pusheen if app push is enabled and the fcm token has been retrieved
        // Otherwise if the token hasn't yet been retrieve,
        // then the first registration with pusheen hasn't occurred so the opted out update should occur at that point
        if (appEnabledPush && !fcmToken.isEmpty()) {
            WorkManager.getInstance().enqueue(PusheenRegistrationWorker.createTask());
        }
    }

    private boolean isPusheenSiteIdInitialized() {
        return !sharedPref.getString(context.getString(R.string.pusheen_site_id_pref_key), "").isEmpty();
    }

    private boolean isPusheenSandyClientIdInitialized() {
        return !sharedPref.getString(context.getString(R.string.pusheen_sandy_client_id_pref_key), "").isEmpty();
    }

    private boolean isNotificationIconResourceIdInitialized() {
        return sharedPref.getInt(context.getString(R.string.icon_resource_id_pref_key), -1) > 0;
    }

    public JSONObject getSubscriptionStatus() throws JSONException {
        JSONObject subscriptionStatus = new JSONObject();

        boolean pushEnabledInApp =
                sharedPref.getBoolean(context.getString(R.string.app_enabled_push_pref_key), false);
        boolean optedOut = sharedPref.getBoolean(context.getString(R.string.opted_out_pref_key), false);

        subscriptionStatus.put(CAN_SUBSCRIBE_KEY, deviceGooglePlayServicesCompatible() && !pushEnabledInApp);
        subscriptionStatus.put(IS_SUBSCRIBED_KEY, pushEnabledInApp);
        subscriptionStatus.put(OPTED_OUT_KEY, optedOut);

        return subscriptionStatus;
    }

    /**
     * Check to see if device supports Google Play Services.
     * @return true if GPS compatible, false otherwise
     */
    public boolean deviceGooglePlayServicesCompatible() {
        final int resultCode = apiAvailability.isGooglePlayServicesAvailable(context);

        if (resultCode == ConnectionResult.SUCCESS) {
            return true;
        }

        if (apiAvailability.isUserResolvableError(resultCode)) {
            return true;
        }

        return false;
    }

    /**
     *
     * @param activity Activity that will be used to display dialog which
     *                 allows user to enable google play services
     */
    public void enablePushNotifications(Activity activity) throws PushException {
        checkPreconditionForEnablingPush();

        if (checkPlayServices(activity)) {
            sendSystemAllowed();
            sharedPref.edit().putBoolean(context.getString(R.string.app_enabled_push_pref_key), true).commit();

            Log.d(TAG, "Push notifications enabled. Starting registration process.");

            // Start registration with FCM server
            FirebaseMessaging.getInstance().setAutoInitEnabled(true);
        } else {
            pushEventListener.systemBlocked(clientId);
        }
    }

    public void enablePushPreviewNotifications(Activity activity) throws PushException {
        checkPreconditionForEnablingPush();

        if (checkPlayServices(activity)) {
            sendSystemAllowed();
            sharedPref.edit()
                    .putBoolean(context.getString(R.string.app_enabled_push_pref_key), true)
                    .putBoolean(context.getString(R.string.app_enabled_preview_pref_key), true)
                    .apply();

            Log.d(TAG, "Preview enabled. Starting registration process.");

            // Start registration with FCM server
            // if token is there
            String fcmToken = sharedPref.getString(context.getString(R.string.fcm_token_pref_key), "");

            if (!fcmToken.isEmpty()) {
                WorkManager.getInstance().enqueue(PusheenRegistrationWorker.createTask());
            } else {
                FirebaseMessaging.getInstance().setAutoInitEnabled(true);
            }
        } else {
            pushEventListener.systemBlocked(clientId);
        }
    }

    private void checkPreconditionForEnablingPush() throws PushException {
        boolean hasSiteId = isPusheenSiteIdInitialized();
        boolean hasSandyId = isPusheenSandyClientIdInitialized();
        if (hasSiteId && hasSandyId && isNotificationIconResourceIdInitialized()) {
            return;
        }

        throw new PushException("Push siteId, sandyClientId and/or notification icon resource have not been initialized.");
    }

    /**
     * Check the device to make sure it has the Google Play Services APK. If
     * it doesn't, display a dialog that allows users to download the APK from
     * the Google Play Store or enable it in the device's system settings.
     */
    private boolean checkPlayServices(final Activity activity) {
        final int resultCode = apiAvailability.isGooglePlayServicesAvailable(activity);

        if (resultCode == ConnectionResult.SUCCESS) {
            return true;
        }

        if (apiAvailability.isUserResolvableError(resultCode)) {
            activity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    apiAvailability.getErrorDialog(activity, resultCode, PLAY_SERVICES_RESOLUTION_REQUEST).show();
                }
            });
        } else {
            Log.i(TAG, "This device does not support google play services. FCM cannot be initialized.");
        }

        return false;
    }

    private String getClientId() {
        String pusheenClientId = sharedPref.getString(context.getString(R.string.pusheen_client_id_pref_key), "");

        if (pusheenClientId.isEmpty()) {
            pusheenClientId = HexUtilities.getRandomHexString(CLIENT_ID_LENGTH_HEX_DIGITS);
            sharedPref.edit().putString(context.getString(R.string.pusheen_client_id_pref_key), pusheenClientId).commit();

            Log.d(TAG, "ClientId generated.");

        }
        Log.d(TAG, "ClientId: " + pusheenClientId);
        return pusheenClientId;
    }

    private void sendSystemAllowed() {
        String systemAllowedSentKey = context.getString(R.string.system_allowed_sent_pref_key);

        boolean systemAllowedSent = sharedPref.getBoolean(systemAllowedSentKey, false);

        if (!systemAllowedSent) {
            sharedPref.edit().putBoolean(systemAllowedSentKey, true).commit();
            pushEventListener.systemAllowed(clientId);
        }
    }
}
