/*
 * Licensed under the License located at push-android-client/LICENSE
 * You may not use this file except in compliance with the License.
 */
package com.mobify.push.pushclient;

import android.content.Context;
import android.content.SharedPreferences;
import android.util.Log;

import com.mobify.push.pushclient.rest.Attributes;
import com.mobify.push.pushclient.rest.PushSubscription;
import com.mobify.push.pushclient.rest.PusheenClient;
import com.mobify.push.pushclient.utilities.LocaleUtilities;

import java.io.IOException;

import retrofit2.Call;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.moshi.MoshiConverterFactory;

public class PusheenRegistrar {
    private static final String TAG = PusheenRegistrar.class.getName();
    private static final String PUSHEEN_DOMAIN = "https://webpush.mobify.net/";
    private static final String SUBSCRIPTION_STATUS = "subscribed";
    private static final String ANDROID_ENDPOINT_NAME = "AndroidPushNotificationsNative";
    private static final String USER_AGENT = "user-agent";
    private static final boolean NOT_A_PREVIEW_DEVICE = false;

    private Context context;
    private SharedPreferences sharedPref;
    protected Retrofit retrofit;
    protected PusheenClient pusheenClient;

    public PusheenRegistrar(Context context) {
        this.context = context;
        this.sharedPref = context.getSharedPreferences(
                context.getString(R.string.push_client_shared_pref_file_key), Context.MODE_PRIVATE);
        this.retrofit =  new Retrofit.Builder()
                .baseUrl(PUSHEEN_DOMAIN)
                .addConverterFactory(MoshiConverterFactory.create())
                .build();

        this.pusheenClient = retrofit.create(PusheenClient.class);
    }

    public void registerToken() throws Exception {
        PushSubscription pushSubscription = createPushSubscription();

        Log.d(TAG, "Attempting to register FCM token with Pusheen. FCM Token: " + pushSubscription.subscription_id + " Preview status: " + pushSubscription.attributes.preview);

        Call<Void> call = pusheenClient.updatePushSubscription(
                getClientId(),
                getSiteId(),
                pushSubscription);
        Response<Void> response = call.execute();

        Log.d(TAG, "Registering FCM token (" + pushSubscription.subscription_id + ") with pusheen was successful: " + response.isSuccessful() + " Code: " + response.code());

        if (!(response.code() == 200 || response.code() == 201)) {
            throw new IOException("Pusheen Server could not process request.");
        }
        sharedPref.edit()
            .putBoolean(context.getString(R.string.pusheen_preview_enabled_pref_key), pushSubscription.attributes.preview)
            .apply();
    }

    protected PushSubscription createPushSubscription() throws Exception {
        String fcmToken = sharedPref.getString(context.getString(R.string.fcm_token_pref_key), "");

        if (fcmToken.isEmpty()) {
            throw new Exception("FCM Token was not set in Shared Preferences. FCM Token can be obtained by running the RegistrationIntentService.");
        }

        Attributes attributes = new Attributes(
                SUBSCRIPTION_STATUS,
                LocaleUtilities.getUtcOffsetInMinutes(),
                LocaleUtilities.getDeviceLanguageAndCountry(),
                USER_AGENT,
                getIsPreviewDevice(),
                getLocale(),
                getSandyClientId());

        boolean optedOut = sharedPref.getBoolean(context.getString(R.string.opted_out_pref_key), false);
        return new PushSubscription(ANDROID_ENDPOINT_NAME, fcmToken, optedOut, attributes);
    }

    private String getClientId() throws Exception {
        String pusheenClientId = sharedPref.getString(context.getString(R.string.pusheen_client_id_pref_key), "");

        if (pusheenClientId.isEmpty()) {
            throw new Exception("ClientId has not been created. Please ensure it was created correctly.");
        }

        return pusheenClientId;
    }

    private String getSiteId() throws Exception {
        String pusheenSiteId = sharedPref.getString(context.getString(R.string.pusheen_site_id_pref_key), "");

        if (pusheenSiteId.isEmpty()) {
            throw new Exception("Pusheen siteId was not accessible from the share preferences. Please ensure it was set correctly.");
        }
        return pusheenSiteId;
    }

    private String getSandyClientId() throws Exception {
        String sandyClientId = sharedPref.getString(context.getString(R.string.pusheen_sandy_client_id_pref_key), "");

        if (sandyClientId.isEmpty()) {
            throw new Exception("Pusheen sandyClientId was not accessible from the share preferences. Please ensure it was set correctly.");
        }
        return sandyClientId;
    }

    private boolean getIsPreviewDevice() {
        return sharedPref.getBoolean(context.getString(R.string.app_enabled_preview_pref_key), NOT_A_PREVIEW_DEVICE);
    }

    private String getLocale() {
        String locale = sharedPref.getString(context.getString(R.string.locale_pref_key), "");

        if (locale.isEmpty()) {
            locale = LocaleUtilities.getDeviceLanguageAndCountry();
        }
        return locale;
    }
}
